# These are the primary portions of the API wrapper runtime
require "action_kit_api/version"
require "action_kit_api/connection"
require "action_kit_api/searchable"
require "action_kit_api/data_model"

# Extensions to the data model for specific information
require "action_kit_api/event"
require "action_kit_api/event_campaign"
require "action_kit_api/user"
require "action_kit_api/page"
require "action_kit_api/action"

# Page types
require "action_kit_api/page_types/event_create"
require "action_kit_api/page_types/event_signup"
require "action_kit_api/page_types/import"
require "action_kit_api/page_types/petition"
require "action_kit_api/page_types/signup"
require "action_kit_api/page_types/unsubscribe"

module ActionKitApi
  @@connection = nil

  # Takes a page object and a user object and a hash of any additional
  # attributes and records the action. Will return a Action object with
  # status information and unique identifiers
  def self.act(page, user, args = {})
    return false unless page.valid? and user.valid?

    # Ensure we have an ActionKit ID before performing the action
    user.save if user.akid.nil?

    # Include the supplied arguments overiding akid and page name if they
    # were supplied with those in the page and user
    act_attrs = args.update({
      "page" => page.name,
      "akid" => user.akid,
    })

    self.raw_act(act_attrs)
  end

  def self.raw_act(*args)
    response = @@connection.call('act', *args)

    ActionKitApi::Action.new(response)
  end

  # Takes username, password, and hostname. This is simply a wrapper to
  # ActionKitAPi::Connection.connect to make the gem a bit easier to use
  def self.connect(username, password, hostname)
    @@connection = ActionKitApi::Connection.new(username, password, hostname)
  end

  def self.connection
    @@connection
  end
end
