require 'xmlrpc/client'

# A few minor tweaks to the XMLRPC standard library. There has to be a better way
# to handle some of what this is doing without going all Chuck Norris on the standard
# library. Once this library has it's full feature set I'll come back to these and dig
# deep into the bowels of XMLRPC and find a way to improve this. Suggestions welcome.
module XMLRPC
  # This seems silly and dangerous right? Overwriting something in the ruby
  # standard libraries? Well ActionKit is doing XMLRPC wrong. This is almost
  # identical to the Ruby standard library version however it drops the type
  # check on the faultCode and the faultString. The XMLRPC standard indicates
  # that yes the faultCode has to be an integer and faultString has to be a
  # string. ActionKit WILL RETURN A STRING AS THE faultCode.
  #
  # References:
  #   XMLRPC Spec:
  #     http://xmlrpc.scripting.com/spec.html
  #   ActionKit (just an example, see second sentence under 'get'
  #     https://roboticdogs.actionkit.com/docs/manual/api/users.html?highlight=faultcode
  module Convert 
    def self.fault(hash)
      if hash.kind_of? Hash and hash.size == 2 and
        hash.has_key? "faultCode" and hash.has_key? "faultString"

        XMLRPC::FaultException.new(hash["faultCode"], hash["faultString"])
      else
        raise "wrong fault-structure: #{hash.inspect}"
      end
    end
  end

  # We're overriding the user agent to make these requests more obviously
  # from this wrapper
  class Client
    remove_const(:USER_AGENT)
    USER_AGENT = "DFA Ruby action_kit_api/#{ActionKitApi::VERSION} ( http://code.democracyforamerica.com/ )"
  end

  # This was needed to prevent an empty response from the remote
  # server from triggering an exception on calls
  module Config
    remove_const(:ENABLE_NIL_PARSER)
    ENABLE_NIL_PARSER = true
  end
end

# This is the connection information for the ActionKit API
# which is used throughout the rest of the Gem (a connection being
# required for ... well every operation ... this is a remote API
# gem :)
module ActionKitApi
  class Connection
    def initialize(user, password, host)
      # Build the arguments for the XMLRPC::Client object
      conn_args = {
          :user => user,
          :password => password,
          :host => host,
          :use_ssl => true,
          :path => "/api"
        }

      @connection = XMLRPC::Client.new_from_hash(conn_args)

      # Unfortunately XMLRPC::Client does not expose it's SSL settings so
      # we need to dive into it's instance methods to set SSL verification
      # and add a CA to authenticate against
      @connection.instance_variable_get("@http").verify_mode = OpenSSL::SSL::VERIFY_PEER
      @connection.instance_variable_get("@http").ca_file = File.join(File.dirname(__FILE__), "actionkit_ca_chain.pem")

      return nil
    end

    def call(command, args)
      # XMLRPC::Client doesn't flush it's IO buffer which occasionally
      # causes a second command on the same connection to fail with an
      # EOFError. Using the _async version of this causes XMLRPC::Client
      # to use a new connection each time eliminating this issue
      @connection.call_async(command, args)
    end

    def version
      self.call("version", {})
    end

  end
end
