require 'action_kit_api'

module ActionKitApi
  class Event < ApiDataModel
    include Searchable
    
    # Required
    attr_accessor :id, :campaign_id, :creator_id
    
    # Other/Active
    attr_accessor :address1, :address2, :attendee_count, :city, :country,
                  :directions, :ends_at, :host_is_confirmed, :is_approved,
                  :is_full, :is_in_past, :is_open_for_signup, :is_private,
                  :latitude, :longitude, :max_attendees, :note_to_attendees,
                  :notes, :phone, :plus4, :postal, :public_description, :region,
                  :starts_at, :state, :status, :status_summary, :title, :venue,
                  :zip

    def initialize(*args)
      @required_attrs = [:campaign_id, :creator_id]
      @read_only_attrs = [:attendee_count, :host_is_confirmed, :is_full, :is_in_past, :is_open_for_signup, :status_summary]

      super
    end

    def signup(user)
      raise "Can't signup to uncreated Event, please save" if self.id.nil?

      event_campaign = ActionKitApi::EventCampaign.find_by_id(self.campaign_id)

      action_attrs = {
        :event_id => self.id,
        :event_signup_ground_rules => 1,
        :akid => user.akid,
        :page => "#{event_campaign.name}_attend"
      }

      ActionKitApi.raw_act(action_attrs)
    end

    def cancel
      raise "Can't cancel to uncreated Event, please save" if self.id.nil?

      self.status = "cancelled"
      self.save
    end
  end 
end

