require 'action_kit_api'

module ActionKitApi
  class EventCampaign < ApiDataModel
    include Searchable

    # Required
    attr_accessor :id, :name, :title

    # Other/Active
    attr_accessor :allow_private, :default_event_size, :default_title,
                  :max_event_size, :public_create_page,
                  :public_search_page, :require_email_confirmation,
                  :require_staff_approval, :show_address1,
                  :show_public_description, :show_state, :show_title,
                  :show_venue, :show_zip, :starts_at, :use_start_date,
                  :use_start_time, :use_title

    def initialize(*args)
      @required_attrs = [:name, :title]
      
      super

      @default_title ||= @title
      @default_event_size ||= 35
      @show_address1 ||= true
    end

    # Requires at a minimum the creator_id
    def create_event(*args)
      raise "EventCampaign needs to be saved before Event creation" if self.id.nil?

      (args[0]).merge!(:campaign_id => self.id)

      event = ActionKitApi::Event.new(*args)
    end

    # Uses public_search so is subject those limitations
    def find_local_events(zip, radius)
      self.public_search(:zip => zip, :radius => radius)
    end

    # Will not return private events, events that are full, deleted, or in the past
    # and doesn't return extra fields
    def public_search(*args)
      (args[0]).merge!(:campaign_id => self.id)
      results = ActionKitApi.connection.call("Event.public_search", *args)

      results.map do |r|
        Event.new(r)
      end

      results
    end

    def stats
      raise "EventCampaign needs to be saved before retrieving stats" if self.id.nil?
      
      ActionKitApi.connection.call("EventCampaign.stats", {:id => self.id})
    end
  end
end
